#!/bin/bash
#
# Build file to set up and run tests

# Change to repo root
cd $(dirname $0)/../../..

# Prepare worker environment to run tests
source kokoro/linux/prepare_build_linux_rc

./tests.sh cpp_distcheck
