// Protocol Buffers - Google's data interchange format
// Copyright 2013 Google Inc.  All rights reserved.
// https://developers.google.com/protocol-buffers/
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//     * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above
// copyright notice, this list of conditions and the following disclaimer
// in the documentation and/or other materials provided with the
// distribution.
//     * Neither the name of Google Inc. nor the names of its
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package com.google.protobuf.nano;

import java.util.HashMap;
import java.util.Map;

/**
 * Utility class for maps support.
 */
public final class MapFactories {
  public static interface MapFactory {
    <K, V> Map<K, V> forMap(Map<K, V> oldMap);
  }

  // NOTE(liujisi): The factory setter is temporarily marked as package private.
  // The way to provide customized implementations of maps for different
  // platforms are still under discussion.  Mark it as private to avoid exposing
  // the API in proto3 alpha release.
  /* public */ static void setMapFactory(MapFactory newMapFactory) {
    mapFactory = newMapFactory;
  }

  public static MapFactory getMapFactory() {
    return mapFactory;
  }

  private static class DefaultMapFactory implements MapFactory {
    public <K, V> Map<K, V> forMap(Map<K, V> oldMap) {
      if (oldMap == null) {
        return new HashMap<K, V>();
      }
      return oldMap;
    }
  }
  private static volatile MapFactory mapFactory = new DefaultMapFactory();

  private MapFactories() {}
}
