<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/protobuf/descriptor.proto

namespace Google\Protobuf\Internal;

/**
 * Protobuf enum <code>Google\Protobuf\Internal</code>
 */
class FieldOptions_CType
{
    /**
     * Default mode.
     *
     * Generated from protobuf enum <code>STRING = 0;</code>
     */
    const STRING = 0;
    /**
     * Generated from protobuf enum <code>CORD = 1;</code>
     */
    const CORD = 1;
    /**
     * Generated from protobuf enum <code>STRING_PIECE = 2;</code>
     */
    const STRING_PIECE = 2;
}

